# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import constants
from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.functional import base


class TestDeletionPolicy(base.BaseSenlinFunctionalTest):

    def setUp(self):
        super(TestDeletionPolicy, self).setUp()
        self.profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, self.profile_id)
        self.cluster_id = utils.create_a_cluster(self, self.profile_id,
                                                 min_size=0, max_size=5,
                                                 desired_capacity=2)
        self.addCleanup(utils.delete_a_cluster, self, self.cluster_id)

    @decorators.attr(type=['functional'])
    @decorators.idempotent_id('bc4af63f-c236-4e8c-a644-d6211c2ec160')
    def test_deletion_policy(self):
        # Create a deletion policy
        spec = constants.spec_deletion_policy
        policy_id = utils.create_a_policy(self, spec)
        del_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, del_policy['id'])

        # Attach deletion policy to cluster
        utils.cluster_attach_policy(self, self.cluster_id, del_policy['id'])
        self.addCleanup(utils.cluster_detach_policy, self, self.cluster_id,
                        del_policy['id'])

        # Scale out cluster
        utils.cluster_scale_out(self, self.cluster_id)

        # Verify scale out result
        cluster = utils.get_a_cluster(self, self.cluster_id)
        self.assertEqual('ACTIVE', cluster['status'])
        self.assertEqual(3, cluster['desired_capacity'])
        self.assertEqual(3, len(cluster['nodes']))

        # Scale in cluster
        utils.cluster_scale_in(self, self.cluster_id)

        # Verify scale in result
        cluster = utils.get_a_cluster(self, self.cluster_id)
        self.assertEqual('ACTIVE', cluster['status'])
        self.assertEqual(2, cluster['desired_capacity'])
        self.assertEqual(2, len(cluster['nodes']))

    @decorators.attr(type=['functional'])
    @decorators.idempotent_id('b08a229f-2cd4-496a-950a-1daff78e4e70')
    def test_deletion_policy_with_hook(self):
        # Create a deletion policy with hook
        spec = constants.spec_deletion_policy_with_hook
        policy_id = utils.create_a_policy(self, spec)
        del_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, del_policy['id'])

        # Attach deletion policy to cluster
        utils.cluster_attach_policy(self, self.cluster_id, del_policy['id'])
        self.addCleanup(utils.cluster_detach_policy, self, self.cluster_id,
                        del_policy['id'])

        # Scale out cluster
        utils.cluster_scale_out(self, self.cluster_id)

        # Verify scale out result
        cluster = utils.get_a_cluster(self, self.cluster_id)
        self.assertEqual('ACTIVE', cluster['status'])
        self.assertEqual(3, cluster['desired_capacity'])
        self.assertEqual(3, len(cluster['nodes']))

        # Scale in cluster
        _, action_id = utils.cluster_scale_in(self, self.cluster_id,
                                              expected_status='WAITING')

        # get action details of scale in action
        action = utils.get_a_action(self, action_id)
        self.assertTrue(1, len(action['depends_on']))

        # get dependent action and check status
        dep_action_id = action['depends_on'][0]
        dep_action = utils.get_a_action(self, dep_action_id)
        self.assertEqual('WAITING_LIFECYCLE_COMPLETION', dep_action['status'])

        # complete lifecycle
        utils.cluster_complete_lifecycle(self, self.cluster_id,
                                         dep_action_id, wait_timeout=10)

        # verify cluster has been scaled in
        cluster = utils.get_a_cluster(self, self.cluster_id)
        self.assertEqual('ACTIVE', cluster['status'])
        self.assertEqual(2, cluster['desired_capacity'])
        self.assertEqual(2, len(cluster['nodes']))

    @decorators.attr(type=['functional'])
    @decorators.idempotent_id('88ea4617-10a6-4005-a641-b9459418661f')
    def test_deletion_policy_with_hook_timeout(self):
        # Create a deletion policy with hook
        spec = constants.spec_deletion_policy_with_hook
        spec['properties']['hooks']['timeout'] = 1

        policy_id = utils.create_a_policy(self, spec)
        del_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, del_policy['id'])

        # Attach deletion policy to cluster
        utils.cluster_attach_policy(self, self.cluster_id, del_policy['id'])
        self.addCleanup(utils.cluster_detach_policy, self, self.cluster_id,
                        del_policy['id'])

        # Scale out cluster
        utils.cluster_scale_out(self, self.cluster_id)

        # Verify scale out result
        cluster = utils.get_a_cluster(self, self.cluster_id)
        self.assertEqual('ACTIVE', cluster['status'])
        self.assertEqual(3, cluster['desired_capacity'])
        self.assertEqual(3, len(cluster['nodes']))

        # Scale in cluster
        _, action_id = utils.cluster_scale_in(self, self.cluster_id,
                                              expected_status='SUCCEEDED',
                                              wait_timeout=10)

        # verify cluster has been scaled in
        cluster = utils.get_a_cluster(self, self.cluster_id)
        self.assertEqual('ACTIVE', cluster['status'])
        self.assertEqual(2, cluster['desired_capacity'])
        self.assertEqual(2, len(cluster['nodes']))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.api import base


class TestClusterActionCompleteLifecycle(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestClusterActionCompleteLifecycle, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)
        self.cluster_id = utils.create_a_cluster(self, profile_id,
                                                 desired_capacity=1)
        self.addCleanup(utils.delete_a_cluster, self, self.cluster_id)

    @decorators.idempotent_id('d1338e7d-c954-4e1f-8601-dae0709e21fb')
    def test_cluster_action_complete_lifecycle(self):
        params = {
            "scale_out": {
                "count": "1"
            }
        }
        # call cluster scale out to get a valid action id
        res = self.client.trigger_action('clusters', self.cluster_id,
                                         params=params)

        # Verify resp code, body and location in headers
        self.assertEqual(202, res['status'])
        self.assertIn('actions', res['location'])

        action_id = res['location'].split('/actions/')[1]
        self.client.wait_for_status('actions', action_id, 'SUCCEEDED')

        # use scale out action id to call complete_lifecycle
        # since a valid action id is needed
        params = {
            "complete_lifecycle": {
                "lifecycle_action_token": action_id
            }
        }
        # Trigger cluster action
        res = self.client.trigger_action('clusters', self.cluster_id,
                                         params=params)

        # Verify resp code, body and location in headers
        self.assertEqual(202, res['status'])
        self.assertIn('actions', res['location'])

        action_id = res['location'].split('/actions/')[1]
        self.client.wait_for_status('actions', action_id, 'SUCCEEDED')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.api import base


class TestProfileTypeOpeList(base.BaseSenlinAPITest):

    @utils.api_microversion('1.4')
    @decorators.idempotent_id('c4216ea0-df06-434d-b51f-bc1b5661c0bb')
    def test_profile_type_operation_list(self):
        res = self.client.list_profile_type_operation('os.nova.server-1.0')

        # Verify resp of profile type operation list API
        self.assertEqual(200, res['status'])
        self.assertIn('change_password', res['body'])
        self.assertIn('description', res['body']['change_password'])
        self.assertIn('parameters', res['body']['change_password'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.api import base


class TestReceiverDelete(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestReceiverDelete, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)

        cluster_id = utils.create_a_cluster(self, profile_id)
        self.addCleanup(utils.delete_a_cluster, self, cluster_id)

        self.receiver_id = utils.create_a_receiver(self, cluster_id,
                                                   'CLUSTER_RESIZE')

    @decorators.idempotent_id('c67cf6c3-2339-4f10-9631-fb7e9f47170f')
    def test_receiver_delete(self):
        # Verify resp of receiver delete API
        res = self.client.delete_obj('receivers', self.receiver_id)
        self.assertEqual(204, res['status'])
        self.assertIsNone(res['body'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions
import time

from senlin_tempest_plugin.common import constants
from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.integration import base

CONF = config.CONF


class TestHealthPolicy(base.BaseSenlinIntegrationTest):
    def setUp(self):
        super(TestHealthPolicy, self).setUp()

        self.spec = utils.create_spec_from_config(
            network_name=data_utils.rand_name('tempest-created-network')
        )
        utils.prepare_and_cleanup_for_nova_server(
            self, '192.168.198.0/24', spec=self.spec
        )
        self.profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, self.profile_id)
        self.cluster_id = utils.create_a_cluster(self, self.profile_id,
                                                 min_size=0, max_size=5,
                                                 desired_capacity=1)
        self.addCleanup(utils.delete_a_cluster, self, self.cluster_id)

    @classmethod
    def skip_checks(cls):
        super(TestHealthPolicy, cls).skip_checks()
        if CONF.clustering.health_policy_version != '1.1':
            skip_msg = ("%s skipped as only Health Policy 1.1 is supported" %
                        cls.__name__)
            raise cls.skipException(skip_msg)

    def _detach_policy(self, policy_id):
        # ignore BadRequest exceptions that are raised because
        # policy is not attached
        try:
            utils.cluster_detach_policy(self, self.cluster_id, policy_id)

            # wait to let health checks stop
            time.sleep(5)
        except exceptions.BadRequest:
            pass

    @decorators.attr(type=['integration'])
    @decorators.idempotent_id('d40ae276-2cbc-4073-b71d-0854ddf8c6a1')
    def test_health_policy(self):
        # Create a health policy
        spec = constants.spec_health_policy
        policy_id = utils.create_a_policy(self, spec)
        del_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, del_policy['id'], True)
        http_server, log_file = utils.start_http_server()
        self.addCleanup(utils.terminate_http_server, http_server, log_file)

        # Attach health policy to cluster
        utils.cluster_attach_policy(self, self.cluster_id, del_policy['id'])
        self.addCleanup(self._detach_policy, del_policy['id'])

        # wait for health checks to run
        time.sleep(5)

        # check that URL was queried for each node as part of health check
        out = utils.terminate_http_server(http_server, log_file)
        self.assertTrue(out.count('GET') == 1)

    def _get_node(self, expected_len, index):
        # get physical id of node (cluster is expected to only contain 1 node)
        raw_nodes = utils.list_nodes(self)
        nodes = {
            n['id']: n['physical_id'] for n in raw_nodes
            if n['cluster_id'] == self.cluster_id
        }
        self.assertTrue(len(nodes) == expected_len)

        return list(nodes.keys())[index], list(nodes.values())[index]

    @decorators.idempotent_id('52f34125-3d6e-4250-9d2e-b619a2905969')
    @decorators.attr(type=['integration'])
    @decorators.idempotent_id('94b0c88a-2cd0-4fff-abbd-331e8369b7b4')
    def test_multiple_detection_modes_any(self):
        # Create a health policy
        spec = constants.spec_health_policy
        spec['properties']['detection']['recovery_conditional'] = 'ANY_FAILED'
        policy_id = utils.create_a_policy(self, spec)
        del_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, del_policy['id'], True)
        http_server, log_file = utils.start_http_server()
        self.addCleanup(utils.terminate_http_server, http_server, log_file)

        # manually shutdown server
        node_id, server_id = self._get_node(1, 0)
        self.compute_client.run_operation_obj(
            'servers', server_id, 'action', {'os-stop': None})

        # verify that server is shutdown
        self.compute_client.wait_for_status('servers', server_id,
                                            'SHUTOFF', 60)

        # Attach health policy to cluster
        utils.cluster_attach_policy(self, self.cluster_id, del_policy['id'])
        self.addCleanup(self._detach_policy, del_policy['id'])

        # wait for health checks to run and recover node
        time.sleep(15)

        # verify that node has been recovered
        self.client.wait_for_status('nodes', node_id, 'ACTIVE', 60)

        # verify that new server is ACTIVE
        old_server_id = server_id
        node_id, server_id = self._get_node(1, 0)
        self.assertNotEqual(old_server_id, server_id)
        self.compute_client.wait_for_status('servers', server_id, 'ACTIVE', 60)

        # verify that old server no longer exists
        self.assertRaises(
            exceptions.NotFound,
            self.compute_client.get_obj, 'servers', old_server_id)

    @decorators.attr(type=['integration'])
    @decorators.idempotent_id('7ce88abb-0a7c-48a8-91ce-59463b75c1e5')
    def test_multiple_detection_modes_all(self):
        # Create a health policy
        spec = constants.spec_health_policy
        spec['properties']['detection']['recovery_conditional'] = 'ALL_FAILED'
        policy_id = utils.create_a_policy(self, spec)
        del_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, del_policy['id'], True)
        http_server, log_file = utils.start_http_server()
        self.addCleanup(utils.terminate_http_server, http_server, log_file)

        # manually shutdown server
        node_id, server_id = self._get_node(1, 0)
        self.compute_client.run_operation_obj(
            'servers', server_id, 'action', {'os-stop': None})

        # verify that server is shutdown
        self.compute_client.wait_for_status('servers', server_id,
                                            'SHUTOFF', 60)

        # Attach health policy to cluster
        utils.cluster_attach_policy(self, self.cluster_id, del_policy['id'])
        self.addCleanup(self._detach_policy, del_policy['id'])

        # wait for health checks to run
        time.sleep(15)

        # verify that node status has not changed
        res = self.client.get_obj('nodes', node_id)
        self.assertEqual(res['body']['status'], 'ACTIVE')

        # verify that server is still stopped
        res = self.compute_client.get_obj('servers', server_id)
        self.assertEqual(res['body']['status'], 'SHUTOFF')

        # check that URL was queried because ALL_FAILED
        # was specified in the policy
        out = utils.terminate_http_server(http_server, log_file)
        self.assertTrue(out.count('GET') >= 0)

        # wait for health checks to run and recover node
        time.sleep(15)

        # verify that node has been recovered
        self.client.wait_for_status('nodes', node_id, 'ACTIVE', 60)

        # verify that new server is ACTIVE
        old_server_id = server_id
        node_id, server_id = self._get_node(1, 0)
        self.assertNotEqual(old_server_id, server_id)
        self.compute_client.wait_for_status('servers', server_id, 'ACTIVE', 60)

        # verify that old server no longer exists
        self.assertRaises(
            exceptions.NotFound,
            self.compute_client.get_obj, 'servers', old_server_id)

    @decorators.attr(type=['integration'])
    @decorators.idempotent_id('e78e0590-2df7-423b-8f71-8037e03598f3')
    def test_multiple_detection_modes_all_poll_url_fail(self):
        # Create a health policy
        spec = constants.spec_health_policy
        spec['properties']['detection']['recovery_conditional'] = 'ALL_FAILED'
        policy_id = utils.create_a_policy(self, spec)
        del_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, del_policy['id'], True)

        # get node_id
        node_id, server_id = self._get_node(1, 0)

        # Attach health policy to cluster without http server running
        utils.cluster_attach_policy(self, self.cluster_id, del_policy['id'])
        self.addCleanup(self._detach_policy, del_policy['id'])

        # wait for health checks to run
        time.sleep(15)

        # verify that node status has not changed
        res = self.client.get_obj('nodes', node_id)
        self.assertEqual(res['body']['status'], 'ACTIVE')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.api import base


class TestReceiverUpdate(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestReceiverUpdate, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)

        cluster_id = utils.create_a_cluster(self, profile_id)
        self.addCleanup(utils.delete_a_cluster, self, cluster_id)

        self.receiver_id = utils.create_a_receiver(self, cluster_id,
                                                   'CLUSTER_SCALE_IN',
                                                   params={"count": 5})
        self.addCleanup(self.client.delete_obj, 'receivers', self.receiver_id)

    @decorators.idempotent_id('fdb2f393-effb-4794-95ac-1f6a119f4474')
    def test_receiver_update(self):
        params = {
            "receiver": {
                "name": "update-receiver-name",
                "action": "CLUSTER_SCALE_IN",
                "params": {
                    "count": "2"
                }
            }
        }
        res = self.client.update_obj('receivers', self.receiver_id, params)

        self.assertEqual(200, res['status'])
        self.assertIsNotNone(res['body'])
        receivers = res['body']

        for key in ['action', 'cluster_id', 'created_at', 'domain', 'id',
                    'name', 'params', 'project', 'type', 'updated_at', 'user']:
            self.assertIn(key, receivers)
        self.assertEqual('update-receiver-name', receivers['name'])
        self.assertEqual({'count': '2'}, receivers['params'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.api import base


class TestNodeShow(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestNodeShow, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)

        self.node_id = utils.create_a_node(self, profile_id)
        self.addCleanup(utils.delete_a_node, self, self.node_id)

    @utils.api_microversion('1.0')
    @decorators.idempotent_id('302372e8-efa2-4348-88dd-8a1829e5e26c')
    def test_node_show(self):
        expected_keys = ['cluster_id', 'created_at', 'data', 'domain',
                         'id', 'index', 'init_at', 'metadata', 'name',
                         'physical_id', 'profile_id', 'profile_name',
                         'project', 'role', 'status', 'status_reason',
                         'updated_at', 'user']
        self._show_node(expected_keys)

    @decorators.idempotent_id('81e65048-c68b-44a1-b351-d572bd68bf06')
    @utils.api_microversion('1.13')
    def test_node_show_v1_13(self):
        expected_keys = ['cluster_id', 'created_at', 'data', 'domain',
                         'id', 'index', 'init_at', 'metadata', 'name',
                         'physical_id', 'profile_id', 'profile_name',
                         'project', 'role', 'status', 'status_reason',
                         'tainted', 'updated_at', 'user']
        self._show_node(expected_keys)

    def _show_node(self, expected_keys):
        res = self.client.get_obj('nodes', self.node_id)

        # Verify resp of node get API
        self.assertEqual(200, res['status'])
        self.assertIsNone(res['location'])
        self.assertIsNotNone(res['body'])
        node = res['body']
        for key in expected_keys:
            self.assertIn(key, node)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as exc
import time

from senlin_tempest_plugin.common import constants
from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.functional import base

CONF = config.CONF


class TestHealthPolicy(base.BaseSenlinFunctionalTest):
    def setUp(self):
        super(TestHealthPolicy, self).setUp()

        self.profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, self.profile_id)
        self.cluster_id = utils.create_a_cluster(self, self.profile_id,
                                                 min_size=0, max_size=5,
                                                 desired_capacity=1)
        self.addCleanup(utils.delete_a_cluster, self, self.cluster_id)

    @classmethod
    def skip_checks(cls):
        super(TestHealthPolicy, cls).skip_checks()
        if CONF.clustering.health_policy_version != '1.1':
            skip_msg = ("%s skipped as only Health Policy 1.1 is supported" %
                        cls.__name__)
            raise cls.skipException(skip_msg)

    @decorators.attr(type=['functional'])
    @decorators.idempotent_id('adfd813c-08c4-4650-9b66-a1a6e63b842e')
    def test_health_policy(self):
        # Create a health policy
        spec = constants.spec_health_policy
        policy_id = utils.create_a_policy(self, spec)
        del_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, del_policy['id'], True)
        http_server, log_file = utils.start_http_server()
        self.addCleanup(utils.terminate_http_server, http_server, log_file)

        def detach_policy():
            # ignore BadRequest exceptions that are raised because
            # policy is not attached
            try:
                utils.cluster_detach_policy(self, self.cluster_id,
                                            del_policy['id'])

                # wait to let health checks stop
                time.sleep(5)
            except exc.BadRequest:
                pass

        # Attach health policy to cluster
        utils.cluster_attach_policy(self, self.cluster_id, del_policy['id'])
        self.addCleanup(detach_policy)

        # wait for health checks to run
        time.sleep(15)

        # check that URL was queried for each node as part of health check
        out = utils.terminate_http_server(http_server, log_file)
        self.assertTrue(out.count('GET') >= 1)

    @decorators.attr(type=['functional'])
    @decorators.idempotent_id('569ca522-00ec-4c1e-b217-4f89d13fe800')
    def test_invalid_health_policy_duplicate_type(self):
        # Create a health policy
        spec = constants.spec_health_policy_duplicate_type
        with self.assertRaisesRegex(
                exc.BadRequest,
                '.*(?i)duplicate detection modes.*'):
            utils.create_a_policy(self, spec)

    @decorators.attr(type=['functional'])
    @decorators.idempotent_id('6f0e0d2c-4381-4afb-ac17-3c2cfed35829')
    def test_invalid_health_policy_invalid_combo(self):
        # Create a health policy
        spec = constants.spec_health_policy_invalid_combo
        with self.assertRaisesRegex(
                exc.BadRequest,
                '.*(?i)invalid detection modes.*'):
            utils.create_a_policy(self, spec)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators
from tempest.lib import exceptions

from senlin_tempest_plugin.tests.api import base


class TestReceiverUpdateNegativeNotFound(base.BaseSenlinAPITest):

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('db22cabd-32af-4045-81e8-c87980f1e9f5')
    def test_receiver_update_receiver_not_found(self):
        params = {
            "receiver": {
                "name": "update-receiver-name",
                "action": "CLUSTER_SCALE_IN",
                "params": {
                    "count": "2"
                }
            }
        }
        self.assertRaises(exceptions.NotFound, self.client.update_obj,
                          'receivers', 'db22cabd-32af-4045-81e8-c87980f1e9f5',
                          params)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('79fcfab6-bb4a-4539-a36e-044f92161062')
    def test_receiver_update_receiver_no_param(self):
        self.assertRaises(exceptions.BadRequest, self.client.update_obj,
                          'receivers', 'db22cabd-32af-4045-81e8-c87980f1e9f5',
                          {})



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
from http import client as http
import http.server as BaseHTTPServer
import os
import signal
import tempfile
import tenacity

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from senlin_tempest_plugin.common import constants

CONF = config.CONF


def is_version_supported(api_version, version):
    """Check if a version is supported by the API.

    :param api_version: Reference endpoint API version.
    :param version: Version to check against API version.
    :return: boolean if the version is supported.
    """

    api_split = list(map(int, api_version.split('.')))
    ver_split = list(map(int, version.split('.')))

    if api_split[0] > ver_split[0] or (
            api_split[0] == ver_split[0] and api_split[1] >= ver_split[1]):
        return True
    return False


def api_microversion(api_microversion):
    """Decorator used to specify api_microversion for test."""
    def decorator(func):
        @functools.wraps(func)
        def wrapped(self):
            if not is_version_supported(self.max_api_version,
                                        api_microversion):
                msg = ('This test case uses a client with API version {0}, '
                       'but Senlin service has max API version {1}.').format(
                    api_microversion,
                    self.max_api_version)
                raise self.skipException(msg)

            old = self.client.api_microversion
            self.client.api_microversion = api_microversion
            try:
                func(self)
            finally:
                self.client.api_microversion = old
        return wrapped
    return decorator


def prepare_and_cleanup_for_nova_server(base, cidr, spec=None):
    keypair_name = create_a_keypair(base, is_admin_manager=False)
    if spec is None:
        base.spec = constants.spec_nova_server
    else:
        base.spec = spec

    base.spec['properties']['key_name'] = keypair_name
    base.addCleanup(delete_a_keypair, base, keypair_name,
                    is_admin_manager=False)

    n_name = base.spec['properties']['networks'][0]['network']
    network_id = create_a_network(base, name=n_name)
    base.addCleanup(delete_a_network, base, network_id)

    subnet_id = create_a_subnet(base, network_id, cidr)
    base.addCleanup(delete_a_subnet, base, subnet_id)


def create_spec_from_config(network_name=None):
    """Utility function that creates a spec object from tempest config"""
    spec = constants.spec_nova_server

    spec['properties']['flavor'] = CONF.compute.flavor_ref
    spec['properties']['image'] = CONF.compute.image_ref

    if network_name:
        spec['properties']['networks'][0]['network'] = network_name

    return spec


def create_a_profile(base, spec=None, name=None, metadata=None):
    """Utility function that generates a Senlin profile."""

    if spec is None:
        spec = constants.spec_nova_server

    if name is None:
        name = data_utils.rand_name("tempest-created-profile")

    if metadata:
        spec['properties']['metadata'] = metadata

    params = {
        'profile': {
            'name': name,
            'spec': spec,
        }
    }
    res = base.client.create_obj('profiles', params)
    return res['body']['id']


def delete_a_profile(base, profile_id, ignore_missing=False):
    """Utility function that deletes a Senlin profile."""
    res = base.client.delete_obj('profiles', profile_id)
    if res['status'] == 404:
        if ignore_missing:
            return
        raise exceptions.NotFound()


def create_a_cluster(base, profile_id, desired_capacity=0, min_size=0,
                     max_size=-1, timeout=None, metadata=None, name=None,
                     config=None, wait_timeout=None):
    """Utility function that generates a Senlin cluster.

    Create a cluster and return it after it is ACTIVE. The function is used for
    deduplicate code in API tests where an 'existing' cluster is needed.
    """
    if name is None:
        name = data_utils.rand_name("tempest-created-cluster")
    params = {
        'cluster': {
            'profile_id': profile_id,
            'desired_capacity': desired_capacity,
            'min_size': min_size,
            'max_size': max_size,
            'timeout': timeout,
            'metadata': metadata,
            'name': name,
            'config': config
        }
    }
    res = base.client.create_obj('clusters', params)
    cluster_id = res['body']['id']
    action_id = res['location'].split('/actions/')[1]
    base.client.wait_for_status('actions', action_id, 'SUCCEEDED',
                                wait_timeout)
    return cluster_id


def update_a_cluster(base, cluster_id, profile_id=None, name=None,
                     expected_status='SUCCEEDED', metadata=None,
                     timeout=None, wait_timeout=None):
    """Utility function that updates a Senlin cluster.

    Update a cluster and return it after it is ACTIVE.
    """
    params = {
        'cluster': {
            'profile_id': profile_id,
            'metadata': metadata,
            'name': name,
            'timeout': timeout
        }
    }
    res = base.client.update_obj('clusters', cluster_id, params)
    action_id = res['location'].split('/actions/')[1]
    base.client.wait_for_status('actions', action_id, expected_status,
                                wait_timeout)
    return res['body']


def get_a_cluster(base, cluster_id, expected_status=None, wait_timeout=None):
    """Utility function that gets a Senlin cluster."""
    if expected_status is None:
        res = base.client.get_obj('clusters', cluster_id)
    else:
        base.client.wait_for_status('clusters', cluster_id, expected_status,
                                    wait_timeout)
        res = base.client.get_obj('clusters', cluster_id)

    return res['body']


def list_clusters(base):
    """Utility function that lists Senlin clusters."""
    res = base.client.list_objs('clusters')
    return res['body']


def _return_last_value(retry_state):
    return retry_state.outcome.result()


@tenacity.retry(
    retry=(tenacity.retry_if_exception_type(exceptions.Conflict) |
           tenacity.retry_if_result(lambda x: x is False)),
    wait=tenacity.wait_fixed(2),
    retry_error_callback=_return_last_value,
    stop=tenacity.stop_after_attempt(5)
)
def delete_a_cluster(base, cluster_id, wait_timeout=None):
    """Utility function that deletes a Senlin cluster."""
    res = base.client.delete_obj('clusters', cluster_id)
    action_id = res['location'].split('/actions/')[1]

    action = base.client.wait_for_status(
        'actions', action_id, ['SUCCEEDED', 'FAILED'], wait_timeout)
    if action['body']['status'] == 'FAILED':
        return False

    base.client.wait_for_delete("clusters", cluster_id, wait_timeout)
    return True


def create_a_node(base, profile_id, cluster_id=None, metadata=None,
                  role=None, name=None, wait_timeout=None):
    """Utility function that creates a node.

    Create a node and return it after it is ACTIVE. This function is for
    minimizing the code duplication that could happen in API tests where
    an 'existing' Senlin node is needed.
    """
    if name is None:
        name = data_utils.rand_name("tempest-created-node")

    params = {
        'node': {
            'profile_id': profile_id,
            'cluster_id': cluster_id,
            'metadata': metadata,
            'role': role,
            'name': name
        }
    }
    res = base.client.create_obj('nodes', params)
    node_id = res['body']['id']
    action_id = res['location'].split('/actions/')[1]
    base.client.wait_for_status('actions', action_id, 'SUCCEEDED',
                                wait_timeout)
    res = base.client.get_obj('nodes', node_id)
    return res['body']['id']


def get_a_node(base, node_id, show_details=False):
    """Utility function that gets a Senlin node."""
    params = None
    if show_details:
        params = {'show_details': True}
    res = base.client.get_obj('nodes', node_id, params)
    return res['body']


def list_nodes(base):
    """Utility function that lists Senlin nodes."""
    res = base.client.list_objs('nodes')
    return res['body']


def update_a_node(base, node_id, profile_id=None, name=None,
                  metadata=None, tainted=None, role=None,
                  wait_timeout=None):
    """Utility function that updates a Senlin node.

    Update a node and return it after it is ACTIVE.
    """
    params = {
        'node': {
            'profile_id': profile_id,
            'metadata': metadata,
            'name': name,
            'role': role
        }
    }
    if tainted is not None:
        params['node']['tainted'] = tainted

    res = base.client.update_obj('nodes', node_id, params)
    action_id = res['location'].split('/actions/')[1]
    base.client.wait_for_status('actions', action_id, 'SUCCEEDED',
                                wait_timeout)

    return res['body']['status_reason']


def delete_a_node(base, node_id, wait_timeout=None):
    """Utility function that deletes a Senlin node."""
    res = base.client.delete_obj('nodes', node_id)
    action_id = res['location'].split('/actions/')[1]
    base.client.wait_for_status('actions', action_id, 'SUCCEEDED',
                                wait_timeout)
    return


def create_a_policy(base, spec=None, name=None):
    """Utility function that generates a Senlin policy."""

    params = {
        'policy': {
            'name': name or data_utils.rand_name("tempest-created-policy"),
            'spec': spec or constants.spec_scaling_policy
        }
    }
    res = base.client.create_obj('policies', params)
    return res['body']['id']


def get_a_policy(base, policy_id):
    """Utility function that gets a Senlin policy."""
    res = base.client.get_obj('policies', policy_id)
    return res['body']


def delete_a_policy(base, policy_id, ignore_missing=False):
    """Utility function that deletes a policy."""
    res = base.client.delete_obj('policies', policy_id)
    if res['status'] == 404:
        if ignore_missing:
            return
        raise exceptions.NotFound()
    return


def get_a_action(base, action_id):
    """Utility function that gets a Senlin action."""
    res = base.client.get_obj('actions', action_id)
    return res['body']


def cluster_attach_policy(base, cluster_id, policy_id,
                          expected_status='SUCCEEDED', wait_timeout=None):
    """Utility function that attach a policy to cluster."""

    params = {
        'policy_attach': {
            'enabled': True,
            'policy_id': policy_id
        }
    }
    res = base.client.trigger_action('clusters', cluster_id, params=params)
    action_id = res['location'].split('/actions/')[1]
    res = base.client.wait_for_status('actions', action_id, expected_status,
                                      wait_timeout)

    return res['body']['status_reason']


@tenacity.retry(
    retry=(tenacity.retry_if_exception_type(exceptions.Conflict) |
           tenacity.retry_if_result(lambda x: x is False)),
    wait=tenacity.wait_fixed(2),
    retry_error_callback=_return_last_value,
    stop=tenacity.stop_after_attempt(5)
)
def cluster_detach_policy(base, cluster_id, policy_id,
                          expected_status='SUCCEEDED', wait_timeout=None):
    """Utility function that detach a policy from cluster."""

    params = {
        'policy_detach': {
            'policy_id': policy_id
        }
    }
    res = base.client.trigger_action('clusters', cluster_id, params=params)
    action_id = res['location'].split('/actions/')[1]

    res = base.client.wait_for_status(
        'actions', action_id, ['SUCCEEDED', 'FAILED'], wait_timeout)
    if res['body']['status'] == 'FAILED':
        return False

    return res['body']['status_reason']


def cluster_replace_nodes(base, cluster_id, nodes,
                          expected_status='SUCCEEDED', wait_timeout=None):
    """Utility function that replace nodes of cluster."""

    params = {
        'replace_nodes': {
            'nodes': nodes
        }
    }
    res = base.client.cluster_replace_nodes('clusters', cluster_id,
                                            params=params)
    action_id = res['location'].split('/actions/')[1]
    res = base.client.wait_for_status('actions', action_id, expected_status,
                                      wait_timeout)
    return res['body']['status_reason']


def cluster_add_nodes(base, cluster_id, nodes, expected_status='SUCCEEDED',
                      wait_timeout=None):
    """Utility function that add nodes to cluster."""

    params = {
        'add_nodes': {
            'nodes': nodes
        }
    }
    res = base.client.trigger_action('clusters', cluster_id, params=params)
    action_id = res['location'].split('/actions/')[1]
    res = base.client.wait_for_status('actions', action_id, expected_status,
                                      wait_timeout)
    return res['body']['status_reason']


def cluster_del_nodes(base, cluster_id, nodes, expected_status='SUCCEEDED',
                      wait_timeout=None):
    """Utility function that delete nodes from cluster."""

    params = {
        'del_nodes': {
            'nodes': nodes
        }
    }
    res = base.client.trigger_action('clusters', cluster_id, params=params)
    action_id = res['location'].split('/actions/')[1]
    res = base.client.wait_for_status('actions', action_id, expected_status,
                                      wait_timeout)
    return res['body']['status_reason']


def cluster_scale_out(base, cluster_id, count=None,
                      expected_status='SUCCEEDED', wait_timeout=None):
    """Utility function that scale out cluster."""

    params = {
        'scale_out': {
            'count': count
        }
    }
    res = base.client.trigger_action('clusters', cluster_id, params=params)
    action_id = res['location'].split('/actions/')[1]
    res = base.client.wait_for_status('actions', action_id, expected_status,
                                      wait_timeout)
    return res['body']['status_reason']


def cluster_scale_in(base, cluster_id, count=None,
                     expected_status='SUCCEEDED', wait_timeout=None):
    """Utility function that scale in cluster."""

    params = {
        'scale_in': {
            'count': count
        }
    }
    res = base.client.trigger_action('clusters', cluster_id, params=params)
    action_id = res['location'].split('/actions/')[1]
    res = base.client.wait_for_status('actions', action_id, expected_status,
                                      wait_timeout)
    return res['body']['status_reason'], action_id


def cluster_resize(base, cluster_id, adj_type=None, number=None, min_size=None,
                   max_size=None, min_step=None, strict=True,
                   expected_status='SUCCEEDED', wait_timeout=None):
    """Utility function that resize cluster."""

    params = {
        'resize': {
            'adjustment_type': adj_type,
            'number': number,
            'min_size': min_size,
            'max_size': max_size,
            'min_step': min_step,
            'strict': strict
        }
    }
    res = base.client.trigger_action('clusters', cluster_id, params=params)
    action_id = res['location'].split('/actions/')[1]
    res = base.client.wait_for_status('actions', action_id, expected_status,
                                      wait_timeout)
    return res['body']['status_reason']


def cluster_complete_lifecycle(base, cluster_id, lifecycle_action_token,
                               expected_status='SUCCEEDED', wait_timeout=None):
    """Utility function that completes lifecycle for a cluster."""

    params = {
        'complete_lifecycle': {
            'lifecycle_action_token': lifecycle_action_token
        }
    }
    res = base.client.trigger_action('clusters', cluster_id, params=params)
    action_id = res['location'].split('/actions/')[1]
    res = base.client.wait_for_status('actions', action_id, expected_status,
                                      wait_timeout)
    return res['body']['status_reason']


def create_a_receiver(base, cluster_id, action, r_type=None, name=None,
                      params=None):
    """Utility function that generates a Senlin receiver."""

    if name is None:
        name = data_utils.rand_name("tempest-created-receiver")

    body = {
        'receiver': {
            'name': name,
            'cluster_id': cluster_id,
            'type': r_type or 'webhook',
            'params': params or {}
        }
    }

    if action is not None:
        body['receiver']['action'] = action

    res = base.client.create_obj('receivers', body)
    return res['body']['id']


def get_a_receiver(base, receiver_id):
    """Utility function that gets a Senlin receiver."""
    res = base.client.get_obj('receivers', receiver_id)
    return res['body']


def delete_a_receiver(base, receiver_id, ignore_missing=False):
    """Utility function that deletes a Senlin receiver."""
    res = base.client.delete_obj('receivers', receiver_id)
    if res['status'] == 404:
        if ignore_missing:
            return
        raise exceptions.NotFound()


def create_a_keypair(base, name=None, is_admin_manager=True):
    """Utility function that creates a Nova keypair."""

    if name is None:
        name = data_utils.rand_name("tempest-created-keypair")

    if is_admin_manager is True:
        body = base.os_admin.keypairs_client.create_keypair(name=name)
        body = body['keypair']
    else:
        params = {
            "keypair": {
                "name": name,
            }
        }
        body = base.compute_client.create_obj('os-keypairs', params)['body']

    return body['name']


def delete_a_keypair(base, name, is_admin_manager=True, ignore_missing=False):
    """Utility function that deletes a Nova keypair."""

    if is_admin_manager is True:
        base.os_admin.keypairs_client.delete_keypair(name)
        return

    res = base.compute_client.delete_obj('os-keypairs', name)
    if res['status'] == 404:
        if ignore_missing is True:
            return
        raise exceptions.NotFound()


def create_a_network(base, name=None):
    """Utility function that creates a Neutron network"""

    if name is None:
        name = data_utils.rand_name("tempest-created-network")

    params = {
        "network": {
            "name": name,
        }
    }
    body = base.network_client.create_obj('networks', params)

    return body['body']['id']


@tenacity.retry(
    retry=tenacity.retry_if_exception_type(exceptions.Conflict),
    wait=tenacity.wait_fixed(5),
    retry_error_callback=_return_last_value,
    stop=tenacity.stop_after_attempt(3)
)
def delete_a_network(base, network_id, ignore_missing=False,
                     wait_timeout=None):
    """Utility function that deletes a Neutron network."""
    res = base.network_client.delete_obj('networks', network_id)
    if res['status'] == 404:
        if ignore_missing:
            return
        raise exceptions.NotFound()

    base.network_client.wait_for_delete('networks', network_id, wait_timeout)


def create_a_subnet(base, network_id, cidr, ip_version=4, name=None):
    """Utility function that creates a Neutron subnet"""

    if name is None:
        name = data_utils.rand_name("tempest-created-subnet")

    params = {
        "subnet": {
            "name": name,
            "network_id": network_id,
            "cidr": cidr,
            "ip_version": ip_version,
        }
    }
    body = base.network_client.create_obj('subnets', params)

    return body['body']['id']


@tenacity.retry(
    retry=tenacity.retry_if_exception_type(exceptions.Conflict),
    wait=tenacity.wait_fixed(5),
    retry_error_callback=_return_last_value,
    stop=tenacity.stop_after_attempt(3)
)
def delete_a_subnet(base, subnet_id, ignore_missing=False, wait_timeout=None):
    """Utility function that deletes a Neutron subnet."""
    res = base.network_client.delete_obj('subnets', subnet_id)
    if res['status'] == 404:
        if ignore_missing:
            return
        raise exceptions.NotFound()

    base.network_client.wait_for_delete('subnets', subnet_id, wait_timeout)


def create_queue(base, queue_name):
    """Utility function that creates Zaqar queue."""
    res = base.messaging_client.create_queue(queue_name)

    if res['status'] != 201 and res['status'] != 204:
        msg = 'Failed in creating Zaqar queue %s' % queue_name
        raise Exception(msg)


def delete_queue(base, queue_name):
    """Utility function that deletes Zaqar queue."""
    res = base.messaging_client.delete_queue(queue_name)

    if res['status'] != 204:
        msg = 'Failed in deleting Zaqar queue %s' % queue_name
        raise Exception(msg)


def list_messages(base, queue_name):
    """Utility function that lists messages in Zaqar queue."""
    res = base.messaging_client.list_messages(queue_name)

    if res['status'] != 200:
        msg = 'Failed in listing messsages for Zaqar queue %s' % queue_name
        raise Exception(msg)

    return res['body']['messages']


def post_messages(base, queue_name, messages):
    """Utility function that posts message(s) to Zaqar queue."""
    res = base.messaging_client.post_messages(queue_name,
                                              {'messages': messages})
    if res['status'] != 201:
        msg = 'Failed in posting messages to Zaqar queue %s' % queue_name
        raise Exception(msg)


def start_http_server(port=5050):
    def _get_http_handler_class(filename):
        class StaticHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):

            def do_GET(self):
                data = b'healthy\n'
                self.send_response(http.OK)
                self.send_header('Content-Length', str(len(data)))
                self.end_headers()
                self.wfile.write(data)
                return

            def log_message(self, fmt, *args):
                msg = fmt % args
                with open(filename, 'a+') as tmp:
                    tmp.write("%s\n" % msg)
                return

        return StaticHTTPRequestHandler

    server_address = ('127.0.0.1', port)
    new_file, filename = tempfile.mkstemp()
    handler_class = _get_http_handler_class(filename)
    httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)

    pid = os.fork()
    if pid == 0:
        httpd.serve_forever()
    else:
        return pid, filename


def terminate_http_server(pid, filename):
    os.kill(pid, signal.SIGKILL)

    if not os.path.isfile(filename):
        return ''

    with open(filename, 'r') as f:
        contents = f.read()

    os.remove(filename)
    return contents



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.api import base


class TestServicesList(base.BaseSenlinAPITest):

    @utils.api_microversion('1.7')
    @decorators.idempotent_id('ff263248-19e2-488d-8fb3-fc09ed3b078e')
    def test_services_list(self):
        res = self.admin_client.list_objs('services')

        self.assertEqual(200, res['status'])
        self.assertIsNotNone(res['body'])
        services = res['body']
        for service in services:
            for key in ['binary', 'host', 'id', 'state',
                        'status', 'topic', 'updated_at']:
                self.assertIn(key, service)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators
from tempest.lib import exceptions

from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.api import base


class TestActionUpdateNegative(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestActionUpdateNegative, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)

        params = {
            'cluster': {
                'profile_id': profile_id,
                'desired_capacity': 0,
                'min_size': 0,
                'max_size': -1,
                'timeout': None,
                'metadata': {},
                'name': 'test-cluster-action-show'
            }
        }
        res = self.client.create_obj('clusters', params)
        self.action_id = res['location'].split('/actions/')[1]
        self.addCleanup(utils.delete_a_cluster, self, res['body']['id'])

        self.client.wait_for_status('actions', self.action_id, 'SUCCEEDED')

    @utils.api_microversion('1.12')
    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('72a123c3-cbce-4452-bb53-d3c939116c2f')
    def test_action_update_not_found(self):
        params = {
            "action": {
                "status": "CANCELLED",
            }
        }
        self.assertRaises(
            exceptions.NotFound, self.client.update_obj, 'actions',
            '26cb0f3a-4e6c-49f6-8475-7cb472933bff', params)

    @utils.api_microversion('1.12')
    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('802b24bc-7004-4b25-8b53-a3709e0ad6ed')
    def test_action_update_status_invalid(self):
        params = {
            "action": {
                "status": "invalid",
            }
        }
        self.assertRaises(
            exceptions.BadRequest, self.client.update_obj, 'actions',
            self.action_id, params)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators
from tempest.lib import exceptions

from senlin_tempest_plugin.tests.api import base


class TestProfileListNegativeBadRequest(base.BaseSenlinAPITest):

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0747310b-6d97-47d3-a7d6-c3d6121cee75')
    def test_profile_list_invalid_params(self):
        self.assertRaises(exceptions.BadRequest,
                          self.client.list_objs,
                          'profiles', {'bogus': 'foo'})

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5dcb1ec1-e870-4e25-a0b8-4b596f0607c0')
    def test_profile_list_limit_not_int(self):
        ex = self.assertRaises(exceptions.BadRequest,
                               self.client.list_objs,
                               'profiles', {'limit': 'not-int'})

        message = ex.resp_body['error']['message']
        self.assertEqual("The value for limit must be an integer: 'not-int'.",
                         str(message))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7e389aa6-039d-4d5d-8d4a-ac33c6d471a3')
    def test_profile_list_global_project_false(self):
        ex = self.assertRaises(exceptions.Forbidden,
                               self.client.list_objs,
                               'profiles', {'global_project': 'True'})

        message = ex.resp_body['error']['message']
        self.assertEqual("You are not authorized to complete this operation.",
                         str(message))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6891a01f-e8ab-4e95-bc74-4260745a8fe5')
    def test_profile_list_global_project_not_bool(self):
        ex = self.assertRaises(exceptions.BadRequest,
                               self.client.list_objs,
                               'profiles', {'global_project': 'not-bool'})

        message = ex.resp_body['error']['message']
        self.assertEqual("Invalid value 'not-bool' specified for "
                         "'global_project'", str(message))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b0bc73b2-dff8-416a-b0e5-8d1389468201')
    def test_profile_list_invalid_sort(self):
        ex = self.assertRaises(exceptions.BadRequest,
                               self.client.list_objs,
                               'profiles', {'sort': 'bad-sort'})

        message = ex.resp_body['error']['message']
        self.assertEqual("Unsupported sort key 'bad-sort' for 'sort'.",
                         str(message))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d84461d1-fc0d-4983-8030-3096cf360d45')
    def test_profile_list_invalid_marker(self):
        ex = self.assertRaises(exceptions.BadRequest,
                               self.client.list_objs,
                               'profiles', {'marker': 'bad-marker'})

        message = ex.resp_body['error']['message']
        self.assertEqual(
            "The value for marker is not a valid UUID: 'bad-marker'.",
            str(message))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from senlin_tempest_plugin.common import constants
from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.integration import base


class TestNovaServerCluster(base.BaseSenlinIntegrationNonAdminTest):

    def setUp(self):
        super(TestNovaServerCluster, self).setUp()
        self.spec = utils.create_spec_from_config(
            network_name=data_utils.rand_name('tempest-created-network')
        )
        utils.prepare_and_cleanup_for_nova_server(
            self, '192.168.199.0/24', spec=self.spec
        )
        self.profile_id = utils.create_a_profile(self, self.spec)
        self.addCleanup(utils.delete_a_profile, self, self.profile_id)

    @decorators.attr(type=['integration'])
    @decorators.idempotent_id('c26eae1c-5c46-4a5f-be63-954d7229c8cc')
    def test_cluster_create_delete(self):
        # Create a cluster
        desired_capacity = 2
        min_size = 1
        max_size = 3
        metadata = {'k1': 'v1'}
        timeout = 600

        self.cluster_id = None

        try:
            self.cluster_id = utils.create_a_cluster(
                self, self.profile_id, desired_capacity, min_size, max_size,
                timeout, metadata)

            # Verify creation result
            cluster = utils.get_a_cluster(self, self.cluster_id)
            self.assertIsNotNone(cluster)
            self.assertEqual(desired_capacity, cluster['desired_capacity'])
            self.assertEqual(desired_capacity, len(cluster['nodes']))
            for nid in cluster['nodes']:
                node = utils.get_a_node(self, nid, show_details=True)
                self.assertEqual('ACTIVE', node['status'])
                self.assertEqual(self.cluster_id, node['cluster_id'])
                self.assertIsNotNone(node['details'])
                self.assertEqual('ACTIVE', node['details']['status'])
                self.assertEqual(self.spec['properties']['flavor'],
                                 node['details']['flavor'])
                self.assertEqual(self.spec['properties']['name'],
                                 node['details']['name'])
                metadata = {
                    'cluster_id': cluster['id'],
                    'cluster_node_id': node['id'],
                    'cluster_node_index': str(node['index'])
                }
                self.assertEqual(metadata, node['details']['metadata'])
        finally:
            # Delete cluster
            if self.cluster_id:
                utils.delete_a_cluster(self, self.cluster_id)

    @decorators.attr(type=['integration'])
    @decorators.idempotent_id('9ac7ed9d-7338-45fb-b749-f67ddeb6caa2')
    def test_lifecycle_hook_message(self):
        self.cluster_id = utils.create_a_cluster(self, self.profile_id,
                                                 min_size=0, max_size=5,
                                                 desired_capacity=2)
        self.addCleanup(utils.delete_a_cluster, self, self.cluster_id)

        # Create a deletion policy with hook
        spec = constants.spec_deletion_policy_with_hook
        policy_id = utils.create_a_policy(self, spec)
        del_policy = utils.get_a_policy(self, policy_id)
        self.addCleanup(utils.delete_a_policy, self, del_policy['id'])

        # Create zaqar queue
        queue_name = spec['properties']['hooks']['params']['queue']
        utils.create_queue(self, queue_name)
        self.addCleanup(utils.delete_queue, self, queue_name)

        # Attach deletion policy to cluster
        utils.cluster_attach_policy(self, self.cluster_id, del_policy['id'])
        self.addCleanup(utils.cluster_detach_policy, self, self.cluster_id,
                        del_policy['id'])

        # Scale out cluster
        utils.cluster_scale_out(self, self.cluster_id)

        # Verify scale out result
        cluster = utils.get_a_cluster(self, self.cluster_id)
        self.assertEqual('ACTIVE', cluster['status'])
        self.assertEqual(3, cluster['desired_capacity'])
        self.assertEqual(3, len(cluster['nodes']))

        # Scale in cluster
        _, action_id = utils.cluster_scale_in(self, self.cluster_id,
                                              expected_status='WAITING')

        # get action details of scale in action
        action = utils.get_a_action(self, action_id)
        self.assertTrue(1, len(action['depends_on']))

        # get lifecycle hook message from zaqar queue
        messages = utils.list_messages(self, queue_name)
        self.assertEqual(1, len(messages))
        lifecycle_hook_message = messages[0]['body']

        # get dependent action and check status
        dep_action_id = action['depends_on'][0]
        dep_action = utils.get_a_action(self, dep_action_id)
        self.assertEqual('WAITING_LIFECYCLE_COMPLETION', dep_action['status'])
        self.assertEqual(dep_action_id,
                         lifecycle_hook_message['lifecycle_action_token'])
        self.assertEqual(dep_action['target'],
                         lifecycle_hook_message['node_id'])

        # complete lifecycle
        utils.cluster_complete_lifecycle(self, self.cluster_id, dep_action_id)

        # verify cluster has been scaled in
        cluster = utils.get_a_cluster(self, self.cluster_id,
                                      expected_status='ACTIVE')
        self.assertEqual(2, cluster['desired_capacity'])
        self.assertEqual(2, len(cluster['nodes']))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.api import base


class TestWebhookTrigger(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestWebhookTrigger, self).setUp()
        profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, profile_id)

        cluster_id = utils.create_a_cluster(self, profile_id)
        self.addCleanup(utils.delete_a_cluster, self, cluster_id)

        params = {'max_size': 2}
        self.receiver_id = utils.create_a_receiver(self, cluster_id,
                                                   'CLUSTER_RESIZE',
                                                   params=params)
        self.addCleanup(self.client.delete_obj, 'receivers', self.receiver_id)

        receiver = self.client.get_obj('receivers', self.receiver_id)
        self.webhook_url = receiver['body']['channel']['alarm_url']

    @decorators.idempotent_id('afd671af-330a-46d6-bdb5-9c50966ab8b5')
    def test_webhook_trigger(self):
        res = self.client.trigger_webhook(self.webhook_url)

        # Verify resp of webhook API
        self.assertEqual(202, res['status'])
        self.assertIn('actions', res['location'])

        # Trigger webhook action
        action_id = res['location'].split('/actions/')[1]
        self.client.wait_for_status('actions', action_id, 'SUCCEEDED')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import constants
from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.integration import base


class TestHeatStackCluster(base.BaseSenlinIntegrationNonAdminTest):

    def setUp(self):
        super(TestHeatStackCluster, self).setUp()
        self.profile_id = utils.create_a_profile(
            self, spec=constants.spec_heat_stack)
        self.addCleanup(utils.delete_a_profile, self, self.profile_id)

    @decorators.attr(type=['integration'])
    @decorators.idempotent_id('c9ba5700-f384-4376-b61b-e2b108ab3a16')
    def test_cluster_create_delete(self):
        # Create a cluster
        desired_capacity = 2
        min_size = 1
        max_size = 3
        metadata = {'k1': 'v1'}
        timeout = 300

        self.cluster_id = None

        try:
            self.cluster_id = utils.create_a_cluster(
                self, self.profile_id, desired_capacity, min_size, max_size,
                timeout, metadata)

            # Verify creation result
            cluster = utils.get_a_cluster(self, self.cluster_id)
            self.assertIsNotNone(cluster)
            self.assertEqual(desired_capacity, cluster['desired_capacity'])
            self.assertEqual(desired_capacity, len(cluster['nodes']))
            for nid in cluster['nodes']:
                node = utils.get_a_node(self, nid)
                self.assertEqual('ACTIVE', node['status'])
                self.assertEqual(self.cluster_id, node['cluster_id'])
        finally:
            # Delete cluster
            if self.cluster_id:
                utils.delete_a_cluster(self, self.cluster_id)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg

service_option = cfg.BoolOpt("senlin",
                             default=True,
                             help="Whether or not senlin is expected to be"
                                  "available")

clustering_group = cfg.OptGroup(name="clustering",
                                title="Clustering Service Options")

ClusteringGroup = [
    cfg.StrOpt("catalog_type",
               default="clustering",
               help="Catalog type of the clustering service."),
    cfg.IntOpt("wait_timeout",
               default=180,
               help="Waiting time for a specific status, in seconds."),
    cfg.StrOpt('min_microversion',
               default=None,
               help="Lower version of the test target microversion range. "
                    "The format is 'X.Y', where 'X' and 'Y' are int values. "
                    "Tempest selects tests based on the range between "
                    "min_microversion and max_microversion. If both values "
                    "are None, Tempest avoids tests which require a "
                    "microversion."),
    cfg.StrOpt('max_microversion',
               default='latest',
               help="Upper version of the test target microversion range. "
                    "The format is 'X.Y'. where 'X' and 'Y' are int values. "
                    "Tempest selects tests based on the range between "
                    "microversion and max_microversion. If both values "
                    "are None, Tempest avoids tests which require a "
                    "microversion."),
    cfg.BoolOpt('delete_with_dependency',
                default=False,
                help="Enables tests that delete clusters with resources such "
                     "as policies or receivers attached to it."),
    cfg.StrOpt('health_policy_version',
               default='1.0',
               help='Supported version of the health policy.'),
]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from senlin_tempest_plugin.common import utils
from senlin_tempest_plugin.tests.api import base


class TestProfileList(base.BaseSenlinAPITest):

    def setUp(self):
        super(TestProfileList, self).setUp()
        self.profile_id = utils.create_a_profile(self)
        self.addCleanup(utils.delete_a_profile, self, self.profile_id)

    @decorators.idempotent_id('329d3026-12f7-4369-845b-05914e2a8678')
    def test_profile_list(self):
        res = self.client.list_objs('profiles')

        # Verify resp of profile list API
        self.assertEqual(200, res['status'])
        self.assertIsNone(res['location'])
        self.assertIsNotNone(res['body'])
        profiles = res['body']
        ids = []
        for profile in profiles:
            for key in ['created_at', 'domain', 'id', 'metadata', 'name',
                        'project', 'spec', 'type', 'updated_at', 'user']:
                self.assertIn(key, profile)
            ids.append(profile['id'])
        self.assertIn(self.profile_id, ids)
